/* 
 *
 *   Note:  There is only one UART in the device, so we must switch between the GPS and 
 *          command line interface.
 */

#include <stdint.h>

#include "nrf.h"
#include "nrf_delay.h"
#include "nrf_gpio.h"

#include "common.h"
#include "uart.h"
#include "gps.h"
#include "uif.h"      // user commmand line interface
#include "gpiote.h"

// Globals
uint32_t uart_interface = UART_GPS_INTERFACE;
uint8_t uart_rx_buff[UART_RX_BUFF_LEN];
uint32_t uart_rx_buff_cnt;

// Externs

//*****************************************************************************
// Description: Sends a character to UART.  Execution is blocked until UART 
//              peripheral reports character to have been send.
// Input:   character to send
// Output:  None
//
void uart_putc(uint8_t cr)
{
  NRF_UART0->TXD = (uint8_t)cr;

  while (NRF_UART0->EVENTS_TXDRDY!=1)
  {
    // Wait for TXD data to be sent
  }

  NRF_UART0->EVENTS_TXDRDY=0;
}

//*****************************************************************************
// Description: Sends a string to UART.
//              Execution is blocked until UART peripheral reports all chars
//              to have been sent.  Maximum string length is 254 characters 
//              including null character in the end.
// Input:   Null terminated string to send.
// Output:  None
//
void uart_putstring(const uint8_t *str)
{
  uint_fast8_t i = 0;
  uint8_t ch = str[i++];
  while (ch != '\0')
  {
    uart_putc(ch);
    ch = str[i++];
  }
}

//*****************************************************************************
// Description: UART RX ISR to handle data from GPS and Command Line Interface
// Input:   None
// Output:  None
//
void UART0_IRQHandler(void)
{
  char cr;

  // Must clear the event flag before reading data or UART may miss next byte!
  NRF_UART0->EVENTS_RXDRDY = 0;
  
  cr = (uint8_t)NRF_UART0->RXD;  
  
  switch(uart_interface)
  {
    case(UART_GPS_INTERFACE):
    {
      // Have we seen a '$' (GPS sentence start)?  If so, always restart.
      if(cr == '$')
      {    
        uart_rx_buff_cnt= 0;
        uart_rx_buff[uart_rx_buff_cnt] = cr;
        uart_rx_buff_cnt++;
        break;
      }
      
      if(uart_rx_buff_cnt > 0)
      {
        uart_rx_buff[uart_rx_buff_cnt] = cr;
        
        // Have we overflowed the buffer?
        if(uart_rx_buff_cnt < UART_RX_BUFF_LEN)
        {
          // No.  Get ready for next character
          uart_rx_buff_cnt++;
        }
        else
        {
          // Yes. Reset the buffer counter
          uart_rx_buff_cnt = 0;
        }
        
        // Check if this is GPS code we want (ex. $GPGLL)
        if(uart_rx_buff_cnt == GPS_CODE_LEN)
        {
          if(gps_check_code(uart_rx_buff) == FALSE)
          {
            uart_rx_buff_cnt = 0;
          }
        }
        else
        {  
          // Have we received an entire sentance? i.e. carriage return terminated)
          if(cr == (uint8_t)0xD)  // Carriage return
          {
            // Yes.
            // Add a NULL to end of string so a strlen can be done.
            uart_rx_buff[uart_rx_buff_cnt] = NULL;            
          
            gps_buff_copy((const char *)uart_rx_buff);
            
            uart_rx_buff_cnt = 0;
          }        
        }
      }
      
      break;
    }
    case(UART_CMD_INTERFACE):
    {
      switch(cr)
      {
        case 0x1B:	// Esc key.  
        case 0x03:	// ^C key.  
        {
          PutPrompt();
          break;
        }
        case '\r':
        case '\n':
        {
          if(!uart_rx_buff_cnt) {
            // A 'carriage return' was entered at prompt
            PutPrompt();
          }else{
            // We have received a command
            uart_rx_buff[uart_rx_buff_cnt] = '\0';
            uart_rx_buff_cnt = 0;
            HandleCmd((char *)uart_rx_buff);
          }
          break;
        }
        default:	// Echo and Store the character
        {
           if(uart_rx_buff_cnt == (UART_RX_BUFF_LEN-1)) {      // exceeded RxData size
             uart_rx_buff_cnt = 0;
             PutPrompt();
           }else{
             uart_putc(cr);                         // Echo character
             uart_rx_buff[uart_rx_buff_cnt++] = cr; // Store character
           }
        }
      } // end switch  

      break;      
    }
    default:
    {
    }
  } // end switch  
}

//*****************************************************************************
// Description: Disable the UART
// Input:   None
// Output:  None
//
void uart_disable(void)
{
  NRF_UART0->ENABLE = (UART_ENABLE_ENABLE_Disabled << UART_ENABLE_ENABLE_Pos);  
  NVIC_DisableIRQ(UART0_IRQn);
  NRF_UART0->EVENTS_RXDRDY = 0;
}

//*****************************************************************************
// Description:  Configure UART to use command line interface GPIO pins
// Input:   None
// Output:  None
//
void uart_command_interface_init(void)
{
  // Disable pin change interrupt
  gpiote_disable();
  
  uart_config(UART_CMD_IF_TX_PIN, UART_CMD_IF_RX_PIN, UART_CMD_INTERFACE);
}

//*****************************************************************************
// Description:  Configure UART to use GPS GPIO pins
// Input:   None
// Output:  None
//
void uart_gps_init(void)
{
  // Enable pin change interrupt
  gpiote_init();

  uart_config(UART_GPS_TX_PIN, UART_GPS_RX_PIN, UART_GPS_INTERFACE);
}

//*****************************************************************************
// Description:  Output a debug string 
// Input:   Null terminated string to send.
// Output:  None
//
void uart_debug_putstring(const uint8_t *str)
{
  uart_command_interface_init();
  
  uart_putstring(str);

  uart_gps_init();
}

//*****************************************************************************
// Description:  Configures UART0
// Input: txd_pin_number - gpio number to be used for UART TXD
//        rxd_pin_number - gpio number to be used for UART RXD
//        if - UART interface to select (GPS or command interface)
// Output:  None
//
void uart_config(uint32_t txd_pin_number, uint32_t rxd_pin_number, uint32_t uart_if)
{

  uart_interface = uart_if;

  uart_rx_buff_cnt = 0;

  // Must disable UART before changing port pins
  uart_disable();   
  
  nrf_gpio_cfg_output(txd_pin_number);
  nrf_gpio_cfg_input(rxd_pin_number, NRF_GPIO_PIN_NOPULL);  

  NRF_UART0->PSELTXD = txd_pin_number;
  NRF_UART0->PSELRXD = rxd_pin_number;

  // Enable interrupts
  NRF_UART0->INTENSET = UART_INTENSET_RXDRDY_Msk;
  NVIC_EnableIRQ(UART0_IRQn);

  NRF_UART0->BAUDRATE         = (UART_BAUDRATE_BAUDRATE_Baud9600 << UART_BAUDRATE_BAUDRATE_Pos);
  NRF_UART0->ENABLE           = (UART_ENABLE_ENABLE_Enabled << UART_ENABLE_ENABLE_Pos);
  NRF_UART0->TASKS_STARTTX    = 1;
  NRF_UART0->TASKS_STARTRX    = 1;
  NRF_UART0->EVENTS_RXDRDY    = 0;
}

// end of file
