/* 
 * Example of entered command:
 * <command> -v 123 -s 456
 * argc = 5
 * argv[0] = -v
 * argv[0] = 123
 * argv[0] = -s
 * argv[0] = 456
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "nrf.h"
#include "nrf_delay.h"
#include "nrf_gpio.h"

#include "common.h"
#include "uif.h"
#include "uart.h"
#include "gpiote.h"

// Globals
char argv[MAX_ARGV][MAX_ARGV_LEN];
uint32_t argc; 
char (*ArgvPtr)[MAX_ARGV_LEN] = argv;
void (*funcPtr)(uint8_t, char *); 

// Externs
extern struct_system_flags system_flags;

// Local Prototypes
uint32_t ParseCmd( char *cmdLine, char (*ArgvPtr)[MAX_ARGV_LEN] );
void * FindCmd(char *fname);
void HelpCmd(void);
void VersionCmd(void);
void QuitCmd(void);

static const cmdEntry cmdTableUser[] = {
  {"v",       1,    (void *)VersionCmd,     "system version info"},
  {"h",       1,    (void *)HelpCmd,        "help"},
  {"q",       1,    (void *)QuitCmd,        "quit user interface"},
  {"\0",      0,    (void *)NULL,            "\0"}    // Always leave this as the LAST ENTRY.
};

//*****************************************************************************
// Description: Parses input string.  Searches command table.  Sets appropriate event.
// Input:   None
// Output:  None
//
void HandleCmd(char *RxData)
{
  // Parse the command string
  argc = ParseCmd(RxData,ArgvPtr);

  // Try to find the function in the command table 
  funcPtr = (void (*)(uint8_t, char *))FindCmd(argv[0]);

  if (funcPtr){
    // Command found
    system_flags.uif_cmd_error = FALSE;
    system_flags.uif_cmd_received = TRUE;
  }else{
    // Command not found. Output Error.
    system_flags.uif_cmd_received = FALSE;
    system_flags.uif_cmd_error = TRUE;
  }
}

//*****************************************************************************
// Description: Parses the command line 
// Inputs:  
// Outputs: argc count
// 
uint32_t ParseCmd( char *cmdLine, char (*ArgvPtr)[MAX_ARGV_LEN] )
{
  uint32_t i, j;
  uint32_t argc=0;
  char *ptr;

  for(i=0;i<=MAX_ARGV;i++)
  {
    j=0;
    ptr=(char *)ArgvPtr;
    while ( (*cmdLine != ' ') && (*cmdLine != '\0') ) {
      *ptr++ = *cmdLine++;
      j++;
    }

    if(*cmdLine == ' '){  
      cmdLine++;       // Only increment on spaces 
    }

    if(j>0){
      *ptr = '\0';     // Tag NULL to end of argument
      argc++;
      ArgvPtr++;
    }else{
      return(argc);
    }
  }  // end for

  return(argc);

}

//*****************************************************************************
// Description: Searches command table for existance of requested function.
// Inputs:  None
// Outputs: None
// 
void * FindCmd(char *fname) 
{
  const cmdEntry *cmdEntryPtr;
  uint32_t len;

  if (fname == 0){
    return(0);
  }

  for (len = strlen(fname), cmdEntryPtr = cmdTableUser; cmdEntryPtr->function != NULL; cmdEntryPtr++)
  {
    // return if tag matches function name (arg[0] string)
    if (len > 0 && len >= cmdEntryPtr->minCmdLen) {
      if (strcmp(cmdEntryPtr->cmdName, fname) == 0) {	
        return(cmdEntryPtr->function);
      }
    }
  }

  return(0);
}

//*****************************************************************************
// Description: Run once per background processor loop to process any user 
//              interface commands.
// Inputs:  None
// Outputs: None
//
void uif_command_check(void)
{
  if(system_flags.uif_cmd_received)
  {
    system_flags.uif_cmd_received = FALSE;
    funcPtr(argc, (char *)argv);          // Call the function.
  }

  if(system_flags.uif_cmd_error){
    system_flags.uif_cmd_error = FALSE;
    uart_putstring((const uint8_t*)CMD_NOT_FOUND_MSG);
    PutPrompt();    
  } 
}

//*****************************************************************************
// Description: Outputs the command line prompt
// Inputs:  None
// Outputs: None
// 
void PutPrompt(void)
{
  uart_putstring(CMD_PROMPT);
}

//*****************************************************************************
// Description: 
// Inputs:  None
// Outputs: None
// 
void HelpCmd(void)
{
  const cmdEntry *tablePtr = cmdTableUser;

  uart_putstring((const uint8_t *)"\n\r");
    
  while(strcmp(tablePtr->cmdName,"\0"))
  {
    uart_putstring((const uint8_t *)&(tablePtr->cmdName));    
    uart_putc('\t');
    uart_putstring((const uint8_t *)&(tablePtr->cmdUsage));

    uart_putstring((const uint8_t *)"\n\r");
    
    tablePtr++;
  }
  
  PutPrompt();
}

//*****************************************************************************
// Description:  Outputs systems version information
// Inputs:  None
// Outputs: None
// 
void VersionCmd(void)
{
  uint8_t buff[30];
/*
  (void)sprintf(buff,"\r\nfw:%1d.%1d", (unsigned int)VERSION_MAJOR, 
                                       (unsigned int)VERSION_MINOR);
*/
  uart_putstring(buff);

  PutPrompt();    
}

//*****************************************************************************
// Description: Quit the command line interface & switch UART back to GPS pins.
// Inputs:  None
// Outputs: None
// 
void QuitCmd(void)
{
  // Re-enable GPIOTE and switch the UART to use the GPS pins
  uart_gps_init();
}

// end of file
